from .mocks_generator import *


def test_generate_mocks():
    first_names_mock, last_names_mock, emails_mock = generate_mocks(10)
    assert len(first_names_mock) == 10
    assert len(last_names_mock) == 10
    assert len(emails_mock) == 10


import pandas as pd
import numpy as np

from email_decomposer.EmailDecomposer import EmailDecomposer

email_decomposer = EmailDecomposer()


class TestEmailDecomposer:

    def test_fuzzily_get_email_full_name(self):
        result = email_decomposer.decompose('sarahk42@gmail.com')
        assert result['first_name'] == 'Sarah'
        result = email_decomposer.decompose('moshe.cohen@gmail.com')
        assert result['first_name'] == 'Moshe' and result['last_name'] == 'Cohen'
        result = email_decomposer.decompose('cohen.moshe@gmail.com')
        assert result['first_name'] == 'Moshe' and result['last_name'] == 'Cohen'
        result = email_decomposer.decompose('cohenmoshe@gmail.com')
        assert result['first_name'] == 'Moshe' and result['last_name'] == 'Cohen'
        result = email_decomposer.decompose('moshe@gmail.com')
        assert result['first_name'] == 'Moshe'
        result = email_decomposer.decompose('cohen@gmail.com')
        assert result['last_name'] == 'Cohen'
        result = email_decomposer.decompose('mcohen@gmail.com')
        assert result['last_name'] == 'Cohen'
        result = email_decomposer.decompose('rbrownlr@gmail.com')
        assert result['first_name'] == ''
        result = email_decomposer.decompose('ray_smith@gmail.com')
        assert result['first_name'] == 'Ray'
        result = email_decomposer.decompose('rbrownlr@gmail.com')
        assert result['last_name'] == ''
        result = email_decomposer.decompose('rbrownlr@gmail.com', True)
        assert result['last_name'] == '' and result['host'] == 'gmail.com'

    def test_fuzzily_get_emails_full_names(self):
        result = email_decomposer.decompose(['sarahk42@gmail.com'])
        assert result.iloc[0]['first_name'] == 'Sarah'
        result = email_decomposer.decompose(['sarahk42@gmail.com'], True)
        assert result.iloc[0]['first_name'] == 'Sarah' and result.iloc[0]['host'] == 'gmail.com'

    def test_decompose(self):
        email_data = email_decomposer.decompose(['johndue@intuit.com'])
        assert len(email_data) > 0

    def test_index(self):
        emails = pd.Series(index=[0, 2], data=['stevensmith@intuit.com', 'johndue@intuit.com'])
        email_data = email_decomposer.decompose(emails)
        assert np.all(email_data.index == emails.index)




import pandas as pd


class FullName:

    def __init__(self, first_name: str, last_name: str) -> None:
        self.first_name = first_name if first_name is None else first_name.title()
        self.last_name = last_name if last_name is None else last_name.title()

    def is_empty(self) -> bool:
        return (self.first_name is None or self.first_name == '') \
               and (self.last_name is None or self.last_name == '')

    def is_full(self):
        return self.first_name is not None and self.first_name != '' \
               and \
               self.last_name is not None and self.last_name != ''

    def to_dict(self) -> dict:
        return {'first_name': self.first_name, 'last_name': self.last_name}

    def to_series(self) -> pd.Series:
        return pd.Series(self.to_dict())





import os
import pandas as pd

curr_path = os.path.dirname(os.path.realpath(__file__)) if '__file__' in globals() else os.getcwd()
emails_mock = pd.read_csv(curr_path + '/mocks/emails.csv', skip_blank_lines=False)['email']
first_names_mock = pd.read_csv(curr_path + '/mocks/first_names.csv', skip_blank_lines=False)['first_name']
last_names_mock = pd.read_csv(curr_path + '/mocks/last_names.csv', skip_blank_lines=False)['last_name']




import names
import pandas as pd
import random
import os
import sys

curr_path = os.path.dirname(os.path.realpath(__file__)) if '__file__' in globals() else os.getcwd()


def mock_first_names(n: int=10000):
    first_names = []
    for i in range(n):
        first_names.append(names.get_first_name())
    return pd.Series(first_names).str.lower().rename('first_name')


def mock_last_names(n: int=10000):
    last_names = []
    for i in range(n):
        last_names.append(names.get_last_name())
    return pd.Series(last_names).str.lower().rename('last_name')


def mock_emails(first_names_mock, last_names_mock, n: int=10000):
    patterns = ['{first_name}_{last_name}@domain.com',
                '{last_name}_{first_name}@domain.com',
                '{first_name}{last_name}@domain.com',
                '{last_name}{first_name}@domain.com',
                '{first_name}a@domain.com',
                'a{last_name}@domain.com']
    emails = []
    for i in range(n):
        pattern = random.choice(patterns)
        emails.append(pattern.format(first_name=first_names_mock.loc[i], last_name=last_names_mock.loc[i],))
    return pd.Series(emails).str.lower().rename('email')


def generate_mocks(n=10000):
    first_names_mock = mock_first_names(n)
    last_names_mock = mock_last_names(n)
    emails_mock = mock_emails(first_names_mock, last_names_mock, n)
    return first_names_mock, last_names_mock, emails_mock


if "pytest" not in sys.modules:
    first_names_mock, last_names_mock, emails_mock = generate_mocks()
    first_names_mock.to_csv('mocks/first_names.csv', index=False)
    last_names_mock.to_csv('mocks/last_names.csv', index=False)
    emails_mock.to_csv('mocks/emails.csv', index=False)


import re
from typing import Sequence

import pandas as pd
from email_decomposer.FullName import FullName

import nltk
import os

nltk.download('words')
nltk.download('wordnet')
lemmatizer = nltk.stem.WordNetLemmatizer()

curr_path = os.path.dirname(os.path.realpath(__file__)) if '__file__' in globals() else os.getcwd()

all_first_names_set = set(pd.read_csv(curr_path + '/resources/dist.male.first.txt', sep=r'\s+', header=None)[0].dropna().str.lower().values) | set(pd.read_csv(curr_path + '/resources/dist.female.first.txt', sep=r'\s+', header=None)[0].dropna().str.lower().values)
all_last_names_set = set(pd.read_csv(curr_path + '/resources/Names_2010Census.txt', sep=',')['name'].dropna().str.lower().values)


class EmailDecomposer:

    def __init__(self) -> None:
        self.emails_full_names_functions = [self.__get_email_full_name_using_sep,
                                            self.__get_email_first_name_perfect_match,
                                            self.__get_email_last_name_perfect_match,
                                            self.__get_email_full_name_perfect_match,
                                            self.__get_email_full_name_perfect_match_reversed,
                                            self.__get_email_last_name_near_perfect_match,
                                            self.__get_email_first_name_near_perfect_match]
        # self.__get_email_first_name_where_last_name_not_found,
        # self.__get_email_last_name_where_first_name_not_found]
        self.words_set = set(nltk.corpus.words.words())

    def fuzzily_get_email_full_name(self, email: str) -> FullName:
        email = email.lower()
        email_prefix = email.split(sep='@', maxsplit=1)[0]
        cleaned = re.sub(r'(?:^[\d]+)|(?:[+.\-_\d]+$)', '', email_prefix)
        lemma = lemmatizer.lemmatize(cleaned)
        if lemma in self.words_set:
            return FullName('', '')
        for func in self.emails_full_names_functions:
            full_name = func(cleaned)
            if full_name is not None and not full_name.is_empty():
                return full_name
        return FullName('', '')

    def __to_series(self, data: Sequence[object], name: str, dtype: object) -> pd.Series:
        if type(data) == pd.Series:
            return data
        return None if data is None else pd.Series(data).reset_index(drop=True).rename(name).astype(dtype)

    def fuzzily_get_emails_full_names(self, emails: Sequence[str]) -> pd.DataFrame:
        emails = self.__to_series(emails, 'email', str)
        emails = emails.str.lower()
        return emails.apply(lambda email: self.fuzzily_get_email_full_name(email).to_series())

    def decompose(self, data: object, get_host=False) -> object:
        if isinstance(data, str):
            return self.str_decompose(data, get_host)
        elif isinstance(data, Sequence) or isinstance(data, pd.Series):
            return self.seq_decompose(data, get_host)
        else:
            return None

    def str_decompose(self, email: str, get_host=False) -> dict:
        result = self.fuzzily_get_email_full_name(email.lower()).to_dict()
        if get_host:
            host = email.split(sep='@', maxsplit=2)[1]
            result['host'] = host
        return result

    def seq_decompose(self, emails: Sequence[str], get_host=False) -> pd.DataFrame:
        original_index = None
        if type(emails) == pd.Series:
            original_index = emails.index
        else:
            emails = pd.Series(emails)
        full_names = self.fuzzily_get_emails_full_names(emails)
        if get_host:
            hosts = emails.str.split(pat='@', n=2, expand=True)[1].rename('host')
            return full_names.join(hosts.to_frame()).set_index(original_index if original_index is not None else emails)
        else:
            return full_names.set_index(original_index if original_index is not None else emails)

    def __get_email_full_name_using_sep(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        split = re.split(r'[+.\-_\d]+', prefix)
        if len(split) > 1:
            first_name_candidate = re.sub(r'(?:^[\d]+)|(?:[\d]+$)', '', split[0])
            if first_name_candidate in all_first_names_set:
                first_name = first_name_candidate
            last_name_candidate = re.sub(r'(?:^[\d]+)|(?:[\d]+$)', '', split[-1])
            if last_name_candidate in all_last_names_set:
                last_name = last_name_candidate
            full_name = FullName(first_name, last_name)
            if not full_name.is_full():
                full_name_reversed = self.__get_email_full_name_using_sep_reversed(prefix)
                if full_name_reversed.is_full():
                    return full_name_reversed
                else:
                    return full_name
        return FullName(first_name, last_name)

    def __get_email_full_name_using_sep_reversed(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        split = re.split(r'[+.\-_\d]+', prefix)
        if len(split) > 1:
            first_name_candidate = re.sub(r'(?:^[\d]+)|(?:[\d]+$)', '', split[1])
            if first_name_candidate in all_first_names_set:
                first_name = first_name_candidate
            last_name_candidate = re.sub(r'(?:^[\d]+)|(?:[\d]+$)', '', split[0])
            if last_name_candidate in all_last_names_set:
                last_name = last_name_candidate
        return FullName(first_name, last_name)

    def __get_email_full_name_perfect_match(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        for i in range(len(prefix) - 3, 2, -1):
            if prefix[:i] in all_first_names_set and prefix[i:] in all_last_names_set:
                first_name = prefix[:i]
                last_name = prefix[i:]
                break
        return FullName(first_name, last_name)

    def __get_email_full_name_perfect_match_reversed(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        for i in range(len(prefix) - 3, 2, -1):
            if prefix[:i] in all_last_names_set and prefix[i:] in all_first_names_set:
                first_name = prefix[i:]
                last_name = prefix[:i]
                break
        return FullName(first_name, last_name)

    def __get_email_first_name_perfect_match(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        if prefix in all_first_names_set:
            first_name = prefix
        return FullName(first_name, last_name)

    def __get_email_last_name_perfect_match(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        if prefix in all_last_names_set:
            last_name = prefix
        return FullName(first_name, last_name)

    def __get_email_last_name_near_perfect_match(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        candidate = '' if len(prefix) < 4 else prefix[1:]
        if candidate in all_last_names_set:
            last_name = candidate
        return FullName(first_name, last_name)

    def __get_email_first_name_near_perfect_match(self, prefix: str) -> FullName:
        first_name = ''
        last_name = ''
        candidate = '' if len(prefix) < 4 else prefix[:-1]
        if candidate in all_first_names_set:
            first_name = candidate
        return FullName(first_name, last_name)



import setuptools, os, sys

from setuptools.command.install import install

VERSION = "0.0.1"

with open('README.md') as f:
    README = f.read()

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

with open("requirements.txt", "r") as f:
    reqs = f.read().split("\n")

with open("test_requirements.txt", "r") as f:
    test_reqs = f.read().split("\n")

setuptools.setup(
    author="Elhanan Mishraky",
    author_email="elhanan_mishraky@intuit.com",
    name="email_decomposer",
    license='Apache License 2.0',
    description='Fuzzy decomposition of email prefix to first/last name',
    version=VERSION,
    long_description=README,
    long_description_content_type='text/markdown',
    url="https://github.com/intuit/email-decomposer",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=reqs,
    setup_requires=["pytest-runner"],
    extras_require={
        'test': test_reqs,
    },
    tests_require=test_reqs,
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)